-- Must be the first line
SIGS = {}

SIGS.name = "SlightlyImprovedGearSets"
SIGS.version = "1.3"
SIGS.configVersion = 1
SIGS.minSwapDelaySecs = 4
SIGS.swapFrequencySecs = 0.5
SIGS.defaults = {
}
SIGS.equipmentSlots = {
	EQUIP_SLOT_CHEST,
	EQUIP_SLOT_FEET,
	EQUIP_SLOT_HAND,
	EQUIP_SLOT_HEAD,
	EQUIP_SLOT_LEGS,
	EQUIP_SLOT_SHOULDERS,
	EQUIP_SLOT_WAIST,
	EQUIP_SLOT_COSTUME,
	EQUIP_SLOT_NECK,
	EQUIP_SLOT_RING1,
	EQUIP_SLOT_RING2,
	EQUIP_SLOT_MAIN_HAND,
	EQUIP_SLOT_OFF_HAND,
	EQUIP_SLOT_BACKUP_MAIN,
	EQUIP_SLOT_BACKUP_OFF,
}
SIGS.queue = { }
SIGS.queueLastAction = 0

-- Create the controls for the configuration pannel
function SIGS.CreateConfiguration()

	local LAM = LibStub("LibAddonMenu-2.0")

	local panelData = {
		type = "panel",
		name = "Gear Sets",
		displayName = "Slightly Improved Gear Sets",
		author = "L8Knight",
		version = SIGS.version,
		registerForDefaults = true,
	}

	LAM:RegisterAddonPanel(SIGS.name.."Config", panelData)

	local controlData = {
		[1] = {
			type = "slider",
			name = "Transparency",
			tooltip = "Transparency value for the experience bar",
			min = 0, max = 10, step = 1,
			getFunc = function() return SIGS.vars.minimumAlpha * 10 end,
			setFunc = function(newValue) SIGS.vars.minimumAlpha = newValue / 10.0 end,
			default = SIGS.defaults.minimumAlpha * 10,
		},
		[2] = {
			type = "checkbox",
			name = "Show Percentage Text",
			tooltip = "Show current experience progress as a percentage",
			getFunc = function() return SIGS.vars.showPercentageText end,
			setFunc = function(newValue) SIGS.vars.showPercentageText = newValue; SIGS.RefreshLabel() end,
			default = SIGS.defaults.showPercentageText,
		},
	}

	LAM:RegisterOptionControls(SIGS.name.."Config", controlData)

end

-- Create a one line text label for placing over an attribute or experience bar
function SIGS.NewButton(name, parent, index)

	local b = WINDOW_MANAGER:CreateControl(name, parent, CT_BUTTON)
	b:SetDimensions(32, 32)

	width = ZO_Character:GetWidth() / 8

	if index == 1 then
		b:SetAnchor(BOTTOMLEFT, parent, TOPLEFT, 0, 24)
		iconBase = '/esoui/art/lfg/lfg_tank'
	elseif index == 2 then
		b:SetAnchor(BOTTOM, parent, TOP, -width, 24)
		iconBase = '/esoui/art/lfg/lfg_healer'
	elseif index == 3 then
		b:SetAnchor(BOTTOM, parent, TOP, width, 24)
		iconBase = '/esoui/art/lfg/lfg_dps'
	else
		b:SetAnchor(BOTTOMRIGHT, parent, TOPRIGHT, 0, 28)
		iconBase = '/esoui/art/inventory/inventory_tabicon_quest'
		b:SetDimensions(32, 36)
	end

	b:SetNormalTexture(iconBase..'_up.dds')
	b:SetPressedTexture(iconBase..'_down.dds')
	b:SetMouseOverTexture(iconBase..'_over.dds')
	b:SetHandler("OnMouseUp", SIGS.OnMouseUp)
	b:SetClickSound("Click")
	b:SetHandler("OnMouseEnter", ZO_Options_OnMouseEnter)
	b:SetHandler("OnMouseExit", ZO_Options_OnMouseExit)
	b.gearSetIndex = index

	if SIGS.TableCount(SIGS.vars.sets[index]) == 0 then
		b.data = { tooltipText = string.format(GetString(SIGS_GEAR_SET_TT_EMPTY), index) }
	else
		b.data = { tooltipText = string.format(GetString(SIGS_GEAR_SET_TT_SWAP_TO), index) }
	end

	return b

end

-- Initializer functions that runs once when the game is loading addons
function SIGS.Initialize(eventCode, addOnName)

	-- Only initialize our own addon
	if SIGS.name ~= addOnName then return end

	-- Load the saved variables
	SIGS.vars = ZO_SavedVars:New("SIGSVars", SIGS.configVersion, nil, SIGS.defaults)
	
	if not SIGS.vars.sets then
		SIGS.vars.sets = { {}, {}, {}, {} }
	end

	SIGS.buttons = { }
	SIGS.buttons[1] = SIGS.NewButton("GearSet1", ZO_Character, 1)
	SIGS.buttons[2] = SIGS.NewButton("GearSet2", ZO_Character, 2)
	SIGS.buttons[3] = SIGS.NewButton("GearSet3", ZO_Character, 3)
	SIGS.buttons[4] = SIGS.NewButton("GearSet4", ZO_Character, 4)

	SIGS.lastSwapTime = GetFrameTimeSeconds()

	-- Create config menu
--	SIGS.CreateConfiguration()

	SIGS.AddRowControls()
	
	
	EVENT_MANAGER:RegisterForEvent("SIGS", EVENT_INVENTORY_SINGLE_SLOT_UPDATE, SIGS.RefreshBackpack)


end

function SIGS.CreateMarker(parent)
	
	local control = parent:GetNamedChild("SIGSMarker")
	if not control then
		control = WINDOW_MANAGER:CreateControl(parent:GetName().."SIGSMarker", parent, CT_TEXTURE)
		control:SetDimensions(32, 32)
		control:SetTexture("/esoui/art/campaign/campaignbrowser_fullpop.dds")
		--control:SetColor(0.9, 0.9, 0.9, 1.0)
		--control:SetColor(0.7, 0.7, 0.6, 1.0)
		control:SetHidden(true)
		control:ClearAnchors()
		control:SetAnchor(LEFT, parent, LEFT)
	end
	return control
	
end

function SIGS.AddSetupHook(view)

	if view and view.dataTypes and view.dataTypes[1] then
		local originalCallback = view.dataTypes[1].setupCallback
		view.dataTypes[1].setupCallback = function(rowControl, slot)
			originalCallback(rowControl, slot)
			SIGS.CreateMarker(rowControl)
		end				
	end

end

function SIGS.RefreshBackpack(view)

	if not view or type(view) ~= "userdata" then return end
	if view:IsHidden() or not view.activeControls or view.isGrid then return end
	--if view:IsHidden() or not view.activeControls then return end
	
	for _,control in pairs(view.activeControls) do
		local bagId = control.dataEntry.data.bagId
		local slotIndex = control.dataEntry.data.slotIndex
		local itemId = GetItemUniqueId(bagId, slotIndex)
		local marker = control:GetNamedChild("SIGSMarker")
		-- For Inventory Grid View, put the lock icon in the center of the control
		--if view.isGrid then
		--	marker:ClearAnchors()
		--	marker:SetAnchor(CENTER, control, CENTER)
		--end
		if SIGS.FindInGearSet(itemId) ~= nil then
			marker:SetHidden(false)
		else
			marker:SetHidden(true)
		end
	end

end

function SIGS.AddRowControls()

	SIGS.AddSetupHook(ZO_PlayerInventoryBackpack)
	SIGS.AddSetupHook(ZO_PlayerBankBackpack)
	SIGS.AddSetupHook(ZO_SmithingTopLevelRefinementPanelInventoryBackpack)
	SIGS.AddSetupHook(ZO_SmithingTopLevelDeconstructionPanelInventoryBackpack)
	SIGS.AddSetupHook(ZO_SmithingTopLevelImprovementPanelInventoryBackpack)

end

function SIGS.UpdateRowControls()

	SIGS.RefreshBackpack(ZO_PlayerInventoryBackpack)
	SIGS.RefreshBackpack(ZO_PlayerBankBackpack)
	SIGS.RefreshBackpack(ZO_SmithingTopLevelRefinementPanelInventoryBackpack)
	SIGS.RefreshBackpack(ZO_SmithingTopLevelDeconstructionPanelInventoryBackpack)
	SIGS.RefreshBackpack(ZO_SmithingTopLevelImprovementPanelInventoryBackpack)

end

function SIGS.OnUpdate()

	SIGS.UpdateRowControls()
	SIGS.ProcessQueue()

end

function SIGS.OnMouseUp(control, mouseButton, mouseUpInside, isControlDown, isAltDown, isShiftDown)

	if not mouseUpInside then
		return
	end
	if mouseButton == 1 then
		SIGS.SwapToSet(control.gearSetIndex)
	elseif mouseButton == 2 and (isControlDown or isAltDown or isShiftDown) then
		SIGS.SaveSet(control.gearSetIndex)
	end

end

function SIGS.FindInGearSet(itemId)

	local itemStr = tostring(itemId)
	for index=1, 4, 1 do
		for _, setItemStr in pairs(SIGS.vars.sets[index]) do
			if itemStr == setItemStr then
				return index
			end
		end
	end
	return nil

end

function SIGS.FindInBags(itemId)

	for bag=0, GetMaxBags(), 1 do
		for slot=0, GetBagSize(bag), 1 do
			local i = GetItemUniqueId(bag, slot)
			if i and i == itemId then
				return bag, slot
			end
		end
	end
	return nil, nil

end

function SIGS.TableCount(obj)

	if not obj then return 0 end
	local c = 0
	for _, _ in pairs(obj) do
		c = c + 1
	end
	return c
	
end

function SIGS.UpdateCurrentSet(index)

	for i,b in pairs(SIGS.buttons) do
		if i == index then
			b.data = { tooltipText = string.format(GetString(SIGS_GEAR_SET_TT_ACTIVE), i) }
		elseif SIGS.TableCount(SIGS.vars.sets[i]) == 0 then
			b.data = { tooltipText = string.format(GetString(SIGS_GEAR_SET_TT_EMPTY), i) }
		else
			b.data = { tooltipText = string.format(GetString(SIGS_GEAR_SET_TT_SWAP_TO), i) }
		end
	end

end

function SIGS.UnequipSlot(gearSlot)

	local icon, slotHasItem, sellPrice, isHeldSlot, isHeldNow, locked = GetEquippedItemInfo(gearSlot)
	if slotHasItem then
		UnequipItem(gearSlot)
	end

end

-- Experimental queue for swapping items with a short delay between each equip
function SIGS.ProcessQueue()

	local now = GetFrameTimeSeconds()
	if now - SIGS.queueLastAction < SIGS.swapFrequencySecs then
		return
	end
	SIGS.queueLastAction = now
	
	for index,info in pairs(SIGS.queue) do
		gearSlot = info[1]
		bag = info[2]
		slot = info[3]
		table.remove(SIGS.queue, index)
		EquipItem(bag, slot, gearSlot)
		return -- only process one at a time
	end

end

function SIGS.SwapToSet(index)

	if SIGS.TableCount(SIGS.vars.sets[index]) == 0 then
		d(string.format(GetString(SIGS_GEAR_SET_IS_EMPTY), index))
		return
	end

	-- This limits swapping to once every X seconds. If the swap happens too fast
	-- and the character is not finished with the weapon draw annimation then the
	-- next swap might be messed up.
	local now = GetFrameTimeSeconds()
	if now - SIGS.lastSwapTime < SIGS.minSwapDelaySecs then
		return
	else
		SIGS.lastSwapTime = now
	end

	for _, gearSlot in pairs(SIGS.equipmentSlots) do
		-- Convert the string back into a number. See comment in SIGS.SaveSet() for
		-- more details.
		itemId = tonumber(SIGS.vars.sets[index][gearSlot])
		if itemId then
			bag, slot = SIGS.FindInBags(itemId)
			if bag and slot then
				--table.insert(SIGS.queue, {gearSlot, bag, slot})
				EquipItem(bag, slot, gearSlot)
			end
		elseif gearSlot ~= EQUIP_SLOT_OFF_HAND and gearSlot ~= EQUIP_SLOT_BACKUP_OFF then
			-- Gear slot is blank so unequip this item (unless its the offhand)
			SIGS.UnequipSlot(gearSlot)
		end
	end

	SIGS.UpdateCurrentSet(index)
	d(string.format(GetString(SIGS_GEAR_SET_EQUIPPED), index))

end

function SIGS.SaveSet(index)

	for _, gearSlot in pairs(SIGS.equipmentSlots) do
		SIGS.vars.sets[index][gearSlot] = ""
		local icon, slotHasItem, sellPrice, isHeldSlot, isHeldNow, locked = GetEquippedItemInfo(gearSlot)
		if slotHasItem then
			local itemId = GetItemUniqueId(0, gearSlot)
			if itemId then
				-- The itemId needs to be converted to a string since it is a 64-bit
				-- value and the current implementation of saved variables does not
				-- correctly save 64-bit numbers to the file.
				SIGS.vars.sets[index][gearSlot] = tostring(itemId)
			else
				SIGS.vars.sets[index][gearSlot] = ""
			end
		else
			SIGS.vars.sets[index][gearSlot] = ""
		end
	end
	
	SIGS.UpdateCurrentSet(index)
	d(string.format(GetString(SIGS_GEAR_SET_SAVED), index))

end

-- Register for the init handler (needs to be declaired after the SIGS.Initialize function)
EVENT_MANAGER:RegisterForEvent("SIGS", EVENT_ADD_ON_LOADED, SIGS.Initialize)
